--
-- Crystal Fragment Proc
--

-- This addon detects when Crystal Fragments triggers the effect of granting the
-- next use of 'Crystal Fragments' as an instant cast ability with 50% cost of magicka.
-- 
-- The addon will display an icon when the effect is triggerer
-- in order to facilitate the player knowing that the effect has procced
-- 
-- The ingame interface settings must be set to always show the action bar in order for the addon to work properly and
-- always show the proc.

CFP = {}
LibStub("AceTimer-3.0"):Embed(CFP)
CFP.version = 1.0
CFP.iconActive = false
CFP.enabled = false
CFP.reset = false
CFP.buffers = {}
CFP.buttonListener = {}
CFP.index = -1
CFP.defaults      = {

  ["IconPositionPoint"]     = {CENTER},
  ["IconRelativePoint"]     = {CENTER},
  ["IconOffsetX"]     = {-150},
  ["IconOffsetY"]     = {100},
 
  }

function CFP.BufferScript( bufferKey , bufferTime )
  
  -- Make sure a function is passed and exists
  if not bufferKey then return end
  
  -- Set a default buffer threshold
  bufferTime = bufferTime or 3
  
  -- Get the current time 
  local currentTime = GetFrameTimeMilliseconds()
  
  -- Check the buffer table to make sure the function is eligible
  if not CFP.buffers[bufferKey] then CFP.buffers[bufferKey] = currentTime end
  if ( currentTime - CFP.buffers[bufferKey] >= bufferTime ) then
    CFP.buffers[bufferKey] = currentTime
    return true
  else
    return false
  end
end

--[[ 
 * Initialization function
 * Runs once, when the add-on is fully loaded
 ]]-- 
function Initialize( self, addOnName )

  if ( addOnName ~= "CrystalFragmentsProc" )  then return end
  CFP.vars = ZO_SavedVars:New( 'CFP_VARS' , math.floor( CFP.version * 100 ) , nil , CFP.defaults )
  CFP.GetHotbar()
  CFP.EventPlayerActivated()
  CFP.wm1 = GetWindowManager()
  CFP.crystalFragmentsICON = CFP.wm1:CreateTopLevelWindow()
  CFP.crystalFragmentsICON:SetDimensions(60,60)
  CFP.crystalFragmentsICON:SetAnchor(CFP.vars["IconPositionPoint"][1],GuiRoot,CFP.vars["IconRelativePoint"][1],CFP.vars["IconOffsetX"][1],CFP.vars["IconOffsetY"][1])
  CFP.crystalFragmentsICON:SetMouseEnabled( true )
  CFP.crystalFragmentsICON:SetMovable( true )
  CFP.crystalFragmentsICON.icon = CFP.wm1:CreateControl(nil, CFP.crystalFragmentsICON, CT_TEXTURE)
  CFP.crystalFragmentsICON.icon:SetTexture('esoui/art/icons/ability_sorcerer_thunderclap.dds')
  CFP.crystalFragmentsICON.icon:SetAnchorFill(CFP.crystalFragmentsICON)
  CFP.crystalFragmentsICON:SetHidden(true)
  CFP.crystalFragmentsICON:SetHandler( "OnMouseUp", function( self ) CFP:SaveAnchor( self ) end )

end

function CFP.CheckCast()
  
  local cfIsInBar = false
  for x = 3,8 do
    if GetSlotName(x) == "Crystal Fragments" then
      CFP.index = x
      cfIsInBar = true
    end
  end
  
  if cfIsInBar then
    for i = 3 , 8 do
      local button = _G["ActionButton"..i.."Button"]
      if( button:GetState() == BSTATE_PRESSED ) then
      
        -- Get the used ability
        local ability = CFP.Hotbar[i]
      
        -- Fire a callback when a spell is cast
        if (not CFP.enabled) then
          CFP.cfCost = CFP.Hotbar[CFP.index].cost
          CALLBACK_MANAGER:FireCallbacks( "CFP_SpellCast" , ability )
        else
          if (ability.name == "Crystal Fragments") then
            CFP.crystalFragmentsICON:SetHidden(true)
            CFP:CancelTimer(CFP.timer)
            CFP.enabled = false
            CFP.iconActive = false
            CALLBACK_MANAGER:FireCallbacks( "CFP_Reset" )
            --d("HIDE!")
          end
        end
      end
    end
  end
end

function CFP.EventWeaponSwap( activeWeaponPair, locked )

  CALLBACK_MANAGER:FireCallbacks( "CFP_Reset" )
  CFP.GetHotbar()
  CFP.UpdateIndicator("WeaponSwap")

end

function CFP.EventReticleHiddenUpdate(event, hidden)

  CALLBACK_MANAGER:FireCallbacks( "CFP_Reset" )
  CFP.GetHotbar()
  CFP.UpdateIndicator("ReticleChanged")

end

function CFP.AbiltyListChanged()

  CALLBACK_MANAGER:FireCallbacks( "CFP_Reset" )  
  CFP.GetHotbar()
  CFP.UpdateIndicator("AbilityListChanged")

end

function CFP.ActionSlotUpdated(slotNum)

  CALLBACK_MANAGER:FireCallbacks( "CFP_Reset" ) 
  CFP.GetHotbar()
  CFP.UpdateIndicator("ActionSlotUpdated")

end

function CFP.ActionSlotAbilitySlotted(newAbilitySlotted)

  CALLBACK_MANAGER:FireCallbacks( "CFP_Reset" )
  CFP.GetHotbar()
  CFP.UpdateIndicator("ActionSlotAbilitySlotted")

end

function CFP.UpdateIndicator(triggeredEvent)

  local cfIsInBarU = false
  for x = 3,8 do
    if GetSlotName(x) == "Crystal Fragments" then
      CFP.index = x
      cfIsInBarU = true
    end
  end
  if (not CFP.enabled and cfIsInBarU) then
    CFP.cfCost = CFP.Hotbar[CFP.index].cost
    --d(CFP.index)
    --d(CFP.cfCost)
    --d(triggeredEvent)
  end
end

function CFP.EventPlayerActivated()

      CALLBACK_MANAGER:RegisterCallback("CFP_SpellCast" , function( ability ) CFP.CanFragmentsProc( ability ) end )

      EVENT_MANAGER:RegisterForEvent(
          "CrystalFragmentsProc", 
          EVENT_ACTIVE_WEAPON_PAIR_CHANGED, CFP.EventWeaponSwap )
      
      EVENT_MANAGER:RegisterForEvent( 
          "CrystalFragmentsProc", 
          EVENT_RETICLE_HIDDEN_UPDATE, CFP.EventReticleHiddenUpdate )

      EVENT_MANAGER:RegisterForEvent( 
          "CrystalFragmentsProc", 
          EVENT_ABILITY_LIST_CHANGED, CFP.AbiltyListChanged )

      EVENT_MANAGER:RegisterForEvent( 
          "CrystalFragmentsProc", 
          EVENT_ACTION_SLOT_UPDATED, CFP.ActionSlotUpdated )

      EVENT_MANAGER:RegisterForEvent( 
          "CrystalFragmentsProc", 
          EVENT_ACTION_SLOT_ABILITY_SLOTTED, CFP.ActionSlotAbilitySlotted )

end

-- Determine whether Crystal Fragments can proc?
function CFP.CanFragmentsProc( ability )

  -- Bail if no ability is passed
  if ( ability == nil ) then return end
  CFP.GetHotbar()
  -- Is Crystal Fragments available?
  for i = 3 , #CFP.Hotbar do
    if ( CFP.Hotbar[i].name == "Crystal Fragments" ) then
      -- Get the button
      CFP.buttonListener = _G["ActionButton"..i.."Button"]
    
      -- Start a listener to wait for the proc
      CFP.buttonListener:SetHandler( "OnUpdate" , function() CFP.ListenForFragments( i , CFP.buttonListener ) end )
      -- Bail out of the loop
      break
    end
  end
end

-- If Crystal Fragments is eligible, listen for it proccing
function CFP.ListenForFragments( slot , button )
  
  if (CFP.iconActive) then return end
  -- Do this every 1ms
  if ( not CFP.BufferScript( 'CFP.ListenForFragments' , 10 ) ) then return end
  
  -- Bail for abilities that are unknown
  if ( CFP.Hotbar[slot] == nil ) then return end

  -- Get the new cost
  CFP.newCost = GetSlotAbilityCost( slot )
  --d(CFP.newCost, "NEW COST")
  CALLBACK_MANAGER:RegisterCallback("CFP_Reset" , function() CFP.reset = true end )
  --d(CFP.reset)
  if CFP.reset then 
    CFP.buttonListener:SetHandler( "OnUpdate" , function() return end )
    CFP.crystalFragmentsICON:SetHidden(true)
    CFP.reset = false
  end
  -- If the new cost is half the old cost or less, assume PROC!
  if ( (CFP.newCost <= ( CFP.cfCost / 2 )) and (CFP.newCost > 0)  ) then
      CFP.buttonListener:SetHandler( "OnUpdate" , function() return end )
      CFP.enabled = true
      CFP.crystalFragmentsICON:SetHidden(false)
      CFP.iconActive = true
      --d("CRYSTAL FRAGMENTS PROC")
      CFP.timer = CFP:ScheduleTimer(function() CFP.crystalFragmentsICON:SetHidden(true) CFP.enabled = false CFP.iconActive = false end, 8)
  end
end

function CFP.GetHotbar()
  
  -- Empty the hotbar
  CFP.Hotbar          = {}
  CFP.Hotbar[1]         = {}
  CFP.Hotbar[2]         = {}

  -- Get the current loadout
  for i = 3, 8 do
    if ( IsSlotUsed(i) ) then
      local name      = GetSlotName(i)
      local cost, cType   = GetSlotAbilityCost(i)
      local slot      = {
        ["slot"]    = i,
        ["name"]    = name,
        ["cost"]    = cost,
      }
      CFP.Hotbar[i]     = slot
    else
      CFP.Hotbar[i]     = {
        ["slot"]    = i,
        ["name"]    = 'unused',
        ["cost"]    = -999,
      }
    end
  end
end

function CFP:SaveAnchor( control )
  
  -- Get the new position
  local isValidAnchor, point, relativeTo, relativePoint, offsetX, offsetY = control:GetAnchor()
  
  -- Save the anchors
  if ( isValidAnchor ) then
    CFP.vars["IconPositionPoint"] = {point}
    CFP.vars["IconRelativePoint"] = {relativePoint}
    CFP.vars["IconOffsetX"] = {offsetX}
    CFP.vars["IconOffsetY"] = {offsetY}
  end
end

function CFP.Update()

  if (GetUnitClass( 'player' ) == "Sorcerer") then
    CFP.CheckCast()
  end

end

EVENT_MANAGER:RegisterForEvent( "CrystalFragmentsProc" , EVENT_ADD_ON_LOADED , Initialize )